@@ -1,15 +1,9 @@
 # Based on https://github.com/KiCad/kicad-source-mirror/blob/master/demos/python_scripts_examples/gen_gerber_and_drill_files_board.py
 import sys
 import os
-import click
+from kikit.pcbnew_compatibility import pcbnew
 from pcbnew import *
 
-@click.command()
-@click.argument("boardfile", type=click.Path(dir_okay=False))
-@click.argument("outputdir", type=click.Path(file_okay=False), default=None)
-def gerber(boardfile, outputdir):
-    gerberImpl(boardfile, outputdir)
-
 fullGerberPlotPlan = [
     # name, id, comment
     ("CuTop", F_Cu, "Top layer"),
@@ -24,13 +18,34 @@ fullGerberPlotPlan = [
     ("CmtUser", Cmts_User, "V-CUT")
 ]
 
+exportSettingsJlcpcb = {
+    "UseGerberProtelExtensions": False,
+    "UseAuxOrigin": True,
+    "ExcludeEdgeLayer": False,
+    "MinimalHeader": False,
+    "NoSuffix": False,
+    "MergeNPTH": True,
+    "ZerosFormat": GENDRILL_WRITER_BASE.DECIMAL_FORMAT,
+}
+
+exportSettingsPcbway = {
+    "UseGerberProtelExtensions": True,
+    "UseAuxOrigin": False,
+    "ExcludeEdgeLayer": True,
+    "MinimalHeader": True,
+    "NoSuffix": True,
+    "MergeNPTH": False,
+    "ZerosFormat": GENDRILL_WRITER_BASE.SUPPRESS_LEADING,
+}
+
+
 def hasCopper(plotPlan):
     for _, layer, _ in plotPlan:
         if layer in [F_Cu, B_Cu]:
             return True
     return False
 
-def gerberImpl(boardfile, outputdir, plot_plan=fullGerberPlotPlan, drilling=True):
+def gerberImpl(boardfile, outputdir, plot_plan=fullGerberPlotPlan, drilling=True, settings=exportSettingsJlcpcb):
     """
     Export board to gerbers.
 
@@ -51,17 +66,17 @@ def gerberImpl(boardfile, outputdir, plot_plan=fullGerberPlotPlan, drilling=True
     popt.SetOutputDirectory(plotDir)
 
     popt.SetPlotFrameRef(False)
-    popt.SetLineWidth(FromMM(0.35))
+    popt.SetSketchPadLineWidth(FromMM(0.35))
     popt.SetAutoScale(False)
     popt.SetScale(1)
     popt.SetMirror(False)
     popt.SetUseGerberAttributes(False)
     popt.SetIncludeGerberNetlistInfo(True)
     popt.SetCreateGerberJobFile(True)
-    popt.SetUseGerberProtelExtensions(False)
-    popt.SetExcludeEdgeLayer(True)
+    popt.SetUseGerberProtelExtensions(settings["UseGerberProtelExtensions"])
+    popt.SetExcludeEdgeLayer(settings["ExcludeEdgeLayer"])
     popt.SetScale(1)
-    popt.SetUseAuxOrigin(True)
+    popt.SetUseAuxOrigin(settings["UseAuxOrigin"])
     popt.SetUseGerberX2format(False)
 
     # This by gerbers only
@@ -79,7 +94,8 @@ def gerberImpl(boardfile, outputdir, plot_plan=fullGerberPlotPlan, drilling=True
             popt.SetSkipPlotNPTH_Pads(False)
 
         pctl.SetLayer(id)
-        pctl.OpenPlotfile(name, PLOT_FORMAT_GERBER, comment)
+        suffix = "" if settings["NoSuffix"] else name
+        pctl.OpenPlotfile(suffix, PLOT_FORMAT_GERBER, comment)
         print('plot {}'.format(pctl.GetPlotFileName()))
         jobfile_writer.AddGbrFile(id, os.path.basename(pctl.GetPlotFileName()))
         if pctl.PlotLayer() == False:
@@ -108,16 +124,17 @@ def gerberImpl(boardfile, outputdir, plot_plan=fullGerberPlotPlan, drilling=True
         drlwriter.SetMapFileFormat(PLOT_FORMAT_PDF)
 
         mirror = False
-        minimalHeader = False
-        offset = wxPoint(0,0)
+        minimalHeader = settings["MinimalHeader"]
+        offset = board.GetDesignSettings().m_AuxOrigin
         # False to generate 2 separate drill files (one for plated holes, one for non plated holes)
         # True to generate only one drill file
-        mergeNPTH = True
+        mergeNPTH = settings["MergeNPTH"]
         drlwriter.SetOptions(mirror, minimalHeader, offset, mergeNPTH)
         drlwriter.SetRouteModeForOvalHoles(False)
 
         metricFmt = True
-        drlwriter.SetFormat(metricFmt)
+        zerosFmt = settings["ZerosFormat"]
+        drlwriter.SetFormat(metricFmt, zerosFmt)
         genDrl = True
         genMap = True
         print('create drill and map files in {}'.format(pctl.GetPlotDirName()))
@@ -137,13 +154,13 @@ def pasteDxfExport(board, plotDir):
     pctl = PLOT_CONTROLLER(board)
     popt = pctl.GetPlotOptions()
 
-    popt.SetOutputDirectory(plotDir)
+    popt.SetOutputDirectory(os.path.abspath(plotDir))
     popt.SetAutoScale(False)
     popt.SetScale(1)
     popt.SetMirror(False)
     popt.SetExcludeEdgeLayer(True)
     popt.SetScale(1)
-    popt.SetDXFPlotUnits(DXF_PLOTTER.DXF_UNIT_MILLIMETERS)
+    popt.SetDXFPlotUnits(DXF_UNITS_MILLIMETERS)
     popt.SetDXFPlotPolygonMode(False)
 
     plot_plan = [
@@ -163,17 +180,7 @@ def pasteDxfExport(board, plotDir):
     pctl.ClosePlot()
     return tuple(output)
 
-@click.command()
-@click.argument("boardfile", type=click.Path(dir_okay=False))
-@click.argument("outputdir", type=click.Path(file_okay=False), default=None)
-def dxf(boardfile, outputdir):
-    """
-    Export board edges and pads to DXF.
-
-    If no output dir is specified, use working directory.
-
-    This command is designed for building 3D printed stencils
-    """
+def dxfImpl(boardfile, outputdir):
     basename = os.path.dirname(boardfile)
     if outputdir:
         plotDir = outputdir
