@@ -1,5 +1,7 @@
-from pcbnew import GetBoard, LoadBoard, FromMM, ToMM, wxPoint, wxRect, wxRectMM, wxPointMM
-import pcbnew
+from kikit import pcbnew_compatibility
+from kikit.pcbnew_compatibility import pcbnew
+from pcbnew import (GetBoard, LoadBoard,
+    FromMM, ToMM, wxPoint, wxRect, wxRectMM, wxPointMM)
 from enum import Enum, IntEnum
 from shapely.geometry import Polygon, MultiPolygon, Point, LineString, box
 from shapely.prepared import prep
@@ -48,7 +50,7 @@ def appendItem(board, item):
     """
     try:
         newItem = item.Duplicate()
-    except TypeError: # Module has overridden the method, cannot be called directly
+    except TypeError: # Footprint has overridden the method, cannot be called directly
         newItem = pcbnew.Cast_to_BOARD_ITEM(item).Duplicate().Cast()
     board.Add(newItem)
 
@@ -59,9 +61,9 @@ def transformArea(board, sourceArea, translate, origin, rotationAngle):
     for drawing in collectItems(board.GetDrawings(), sourceArea):
         drawing.Rotate(origin, rotationAngle)
         drawing.Move(translate)
-    for module in collectItems(board.GetModules(), sourceArea):
-        module.Rotate(origin, rotationAngle)
-        module.Move(translate)
+    for footprint in collectItems(board.GetFootprints(), sourceArea):
+        footprint.Rotate(origin, rotationAngle)
+        footprint.Move(translate)
     for track in collectItems(board.GetTracks(), sourceArea):
         track.Rotate(origin, rotationAngle)
         track.Move(translate)
@@ -152,7 +154,7 @@ def undoTransformation(point, rotation, origin, translation):
     return the original point position.
     """
     # Abuse PcbNew to do so
-    segment = pcbnew.DRAWSEGMENT()
+    segment = pcbnew.PCB_SHAPE()
     segment.SetShape(STROKE_T.S_SEGMENT)
     segment.SetStart(wxPoint(point[0], point[1]))
     segment.SetEnd(wxPoint(0, 0))
@@ -160,16 +162,16 @@ def undoTransformation(point, rotation, origin, translation):
     segment.Rotate(origin, -rotation)
     return segment.GetStart()
 
-def removeCutsFromModule(module):
+def removeCutsFromFootprint(footprint):
     """
-    Find all graphical items in the module, remove them and return them as a
+    Find all graphical items in the footprint, remove them and return them as a
     list
     """
     edges = []
-    for edge in module.GraphicalItems():
+    for edge in footprint.GraphicalItems():
         if edge.GetLayerName() != "Edge.Cuts":
             continue
-        module.Remove(edge)
+        footprint.Remove(edge)
         edges.append(edge)
     return edges
 
@@ -182,17 +184,20 @@ def renameNets(board, renamer):
     netinfo = board.GetNetInfo()
 
     newNetMapping = { "": netinfo.GetNetItem("") }
+    newNames = set()
     for name in originalNetNames:
-        newNet = pcbnew.NETINFO_ITEM(board, renamer(name))
+        newName = renamer(name)
+        newNet = pcbnew.NETINFO_ITEM(board, newName)
         newNetMapping[name] = newNet
         board.Add(newNet)
+        newNames.add(newName)
 
     remapNets(board.GetPads(), newNetMapping)
     remapNets(board.GetTracks(), newNetMapping)
     remapNets(board.Zones(), newNetMapping)
 
     for name in originalNetNames:
-        if name != "":
+        if name != "" and name not in newNames:
             board.RemoveNative(netinfo.GetNetItem(name))
 
 def renameRefs(board, renamer):
@@ -200,9 +205,9 @@ def renameRefs(board, renamer):
     Given a board and renaming function (taking original name, returning new
     name) renames the references
     """
-    for module in board.GetModules():
-        ref = module.Reference().GetText()
-        module.Reference().SetText(renamer(ref))
+    for footprint in board.GetFootprints():
+        ref = footprint.Reference().GetText()
+        footprint.Reference().SetText(renamer(ref))
 
 def isBoardEdge(edge):
     """
@@ -210,7 +215,7 @@ def isBoardEdge(edge):
 
     The rule is: all drawings on Edge.Cuts layer are edges.
     """
-    return isinstance(edge, pcbnew.DRAWSEGMENT) and edge.GetLayerName() == "Edge.Cuts"
+    return isinstance(edge, pcbnew.PCB_SHAPE) and edge.GetLayerName() == "Edge.Cuts"
 
 def increaseZonePriorities(board, amount=1):
     for zone in board.Zones():
@@ -234,7 +239,7 @@ class Panel:
         self.hVCuts = set() # Keep V-cuts as numbers and append them just before saving
         self.vVCuts = set() # to make them truly span the whole panel
         self.copperLayerCount = None
-        self.zonesToRefill = pcbnew.ZONE_CONTAINERS()
+        self.zonesToRefill = pcbnew.ZONES()
 
     def save(self, filename):
         """
@@ -256,6 +261,32 @@ class Panel:
     def _uniquePrefix(self):
         return "Board_{}-".format(self.boardCounter)
 
+    def inheritDesignSettings(self, boardFilename):
+        """
+        Inherit design settings from the given board specified by a filename
+        """
+        b = pcbnew.LoadBoard(boardFilename)
+        self.setDesignSettings(b.GetDesignSettings())
+
+    def setDesignSettings(self, designSettings):
+        """
+        Set design settings
+        """
+        self.board.SetDesignSettings(designSettings)
+
+    def inheritProperties(self, boardFilename):
+        """
+        Inherit text properties from a board specified by a properties
+        """
+        b = pcbnew.LoadBoard(boardFilename)
+        self.setDesignSettings(b.GetDesignSettings())
+
+    def setProperties(self, properties):
+        """
+        Set text properties cached in the board
+        """
+        self.board.SetProperties(designSettings)
+
     def appendBoard(self, filename, destination, sourceArea=None,
                     origin=Origin.Center, rotationAngle=0, shrink=False,
                     tolerance=0, bufferOutline=fromMm(0.001), netRenamer=None,
@@ -310,16 +341,16 @@ class Panel:
             renameRefs(board, lambda x: refRenamer(self.boardCounter, x))
 
         drawings = collectItems(board.GetDrawings(), enlargedSourceArea)
-        modules = collectItems(board.GetModules(), enlargedSourceArea)
+        footprints = collectFootprints(board.GetFootprints(), enlargedSourceArea)
         tracks = collectItems(board.GetTracks(), enlargedSourceArea)
         zones = collectItems(board.Zones(), enlargedSourceArea)
 
         edges = []
-        for module in modules:
-            module.Rotate(originPoint, rotationAngle)
-            module.Move(translation)
-            edges += removeCutsFromModule(module)
-            appendItem(self.board, module)
+        for footprint in footprints:
+            footprint.Rotate(originPoint, rotationAngle)
+            footprint.Move(translation)
+            edges += removeCutsFromFootprint(footprint)
+            appendItem(self.board, footprint)
         for track in tracks:
             track.Rotate(originPoint, rotationAngle)
             track.Move(translation)
@@ -377,23 +408,23 @@ class Panel:
     def _setVCutLabelStyle(self, label, layer):
         label.SetText("V-CUT")
         label.SetLayer(layer)
-        label.SetThickness(fromMm(0.4))
+        label.SetTextThickness(fromMm(0.4))
         label.SetTextSize(pcbnew.wxSizeMM(2, 2))
         label.SetHorizJustify(EDA_TEXT_HJUSTIFY_T.GR_TEXT_HJUSTIFY_LEFT)
 
     def _renderVCutV(self, layer=Layer.Cmts_User):
-        """ return list of DRAWSEGMENT V-Cuts """
+        """ return list of PCB_SHAPE V-Cuts """
         bBox = self.boardSubstrate.boundingBox()
         minY, maxY = bBox.GetY() - fromMm(3), bBox.GetY() + bBox.GetHeight() + fromMm(3)
         segments = []
         for cut in self.vVCuts:
-            segment = pcbnew.DRAWSEGMENT()
+            segment = pcbnew.PCB_SHAPE()
             self._setVCutSegmentStyle(segment, layer)
             segment.SetStart(pcbnew.wxPoint(cut, minY))
             segment.SetEnd(pcbnew.wxPoint(cut, maxY))
             segments.append(segment)
 
-            label = pcbnew.TEXTE_PCB(segment)
+            label = pcbnew.PCB_TEXT(segment)
             self._setVCutLabelStyle(label, layer)
             label.SetPosition(wxPoint(cut, minY - fromMm(3)))
             label.SetTextAngle(900)
@@ -401,18 +432,18 @@ class Panel:
         return segments
 
     def _renderVCutH(self, layer=Layer.Cmts_User):
-        """ return list of DRAWSEGMENT V-Cuts """
+        """ return list of PCB_SHAPE V-Cuts """
         bBox = self.boardSubstrate.boundingBox()
         minX, maxX = bBox.GetX() - fromMm(3), bBox.GetX() + bBox.GetWidth() + fromMm(3)
         segments = []
         for cut in self.hVCuts:
-            segment = pcbnew.DRAWSEGMENT()
+            segment = pcbnew.PCB_SHAPE()
             self._setVCutSegmentStyle(segment, layer)
             segment.SetStart(pcbnew.wxPoint(minX, cut))
             segment.SetEnd(pcbnew.wxPoint(maxX, cut))
             segments.append(segment)
 
-            label = pcbnew.TEXTE_PCB(segment)
+            label = pcbnew.PCB_TEXT(segment)
             self._setVCutLabelStyle(label, layer)
             label.SetPosition(wxPoint(maxX + fromMm(3), cut))
             segments.append(label)
@@ -659,7 +690,7 @@ class Panel:
                                     sourceArea, tolerance, verSpace, horSpace,
                                     rotation, netRenamer, refRenamer)
         gridDest = wxPoint(boardSize.GetX(), boardSize.GetY())
-        panelSize = wxRect(destination[0], destination[1],
+        panelSize = wxRect(gridDest[0], gridDest[1],
                        cols * boardSize.GetWidth() + (cols - 1) * horSpace,
                        rows * boardSize.GetHeight() + (rows - 1) * verSpace)
 
@@ -819,7 +850,10 @@ class Panel:
         module = pcbnew.PCB_IO().FootprintLoad(KIKIT_LIB, "Fiducial")
         module.SetPosition(position)
         if(bottom):
-            module.Flip(position)
+            if pcbnew_compatibility.isV6(pcbnew_compatibility.pcbnewVersion):
+                module.Flip(position, False)
+            else:
+                module.Flip(position)
         for pad in module.Pads():
             pad.SetSize(pcbnew.wxSize(copperDiameter, copperDiameter))
             pad.SetLocalSolderMaskMargin(int((openingDiameter - copperDiameter) / 2))
@@ -866,7 +900,7 @@ class Panel:
 
     def layerToTabs(self, layerName, tabWidth):
         """
-        Take all line drawsegments from the given layer and convert them to
+        Take all line pcbshapes from the given layer and convert them to
         tabs.
 
         The tabs are created by placing a tab origin into the line starting
@@ -879,7 +913,7 @@ class Panel:
         Returns list of tabs substrates and a list of cuts to perform.
         """
         lines = [element for element in self.board.GetDrawings()
-                    if isinstance(element, pcbnew.DRAWSEGMENT) and
+                    if isinstance(element, pcbnew.PCB_SHAPE) and
                        element.GetShape() == STROKE_T.S_SEGMENT and
                        element.GetLayerName() == layerName]
         tabs, cuts = [], []
@@ -915,7 +949,7 @@ class Panel:
             raise RuntimeError("The substrate has to be a single piece to fill unused areas")
         increaseZonePriorities(self.board)
 
-        zoneContainer = pcbnew.ZONE_CONTAINER(self.board)
+        zoneContainer = pcbnew.ZONE(self.board)
         boundary = self.boardSubstrate.exterior().boundary
         zoneContainer.Outline().AddOutline(linestringToKicad(boundary))
         for substrate in self.substrates:
@@ -930,4 +964,4 @@ class Panel:
         zoneContainer = zoneContainer.Duplicate()
         zoneContainer.SetLayer(Layer.B_Cu)
         self.board.Add(zoneContainer)
-        self.zonesToRefill.append(zoneContainer)
\ No newline at end of file
+        self.zonesToRefill.append(zoneContainer)
