@@ -1,9 +1,9 @@
-import pcbnew
+from kikit.pcbnew_compatibility import pcbnew
 from pcbnew import wxPoint
 import numpy as np
 from kikit.common import *
 from kikit.defs import *
-from kikit.substrate import Substrate
+from kikit.substrate import Substrate, extractRings, toShapely, linestringToKicad
 from kikit.export import gerberImpl, pasteDxfExport
 import solid
 import solid.utils
@@ -12,7 +12,6 @@ from kikit.common import removeComponents, parseReferences
 
 from shapely.geometry import Point
 
-import click
 
 OUTER_BORDER = fromMm(7.5)
 INNER_BORDER = fromMm(5)
@@ -26,7 +25,7 @@ def addBottomCounterpart(board, item):
     board.Add(item)
 
 def addRoundedCorner(board, center, start, end, thickness):
-    corner = pcbnew.DRAWSEGMENT()
+    corner = pcbnew.PCB_SHAPE()
     corner.SetShape(STROKE_T.S_ARC)
     corner.SetCenter(wxPoint(center[0], center[1]))
     corner.SetArcStart(wxPoint(start[0], start[1]))
@@ -40,7 +39,7 @@ def addRoundedCorner(board, center, start, end, thickness):
     addBottomCounterpart(board, corner)
 
 def addLine(board, start, end, thickness):
-    line = pcbnew.DRAWSEGMENT()
+    line = pcbnew.PCB_SHAPE()
     line.SetShape(STROKE_T.S_SEGMENT)
     line.SetStart(wxPoint(start[0], start[1]))
     line.SetEnd(wxPoint(end[0], end[1]))
@@ -120,7 +119,7 @@ def addFrame(board, rect, bridgeWidth, bridgeSpacing, clearance):
             addBite(board, wxPoint(x2, end), wxPoint(0, 1), wxPoint(-1, 0), clearance)
 
 def addHole(board, position, radius):
-    circle = pcbnew.DRAWSEGMENT()
+    circle = pcbnew.PCB_SHAPE()
     circle.SetShape(STROKE_T.S_CIRCLE)
     circle.SetCenter(wxPoint(position[0], position[1]))
     circle.SetArcStart(wxPoint(position[0], position[1]) + wxPoint(radius/2, 0))
@@ -226,13 +225,11 @@ def makeRegister(board, jigFrameSize, jigThickness, pcbThickness,
 
     innerRing = createOffsetPolygon(board, - innerBorder).exterior.coords
     if topSide:
-        print("top")
         innerRing = mirrorX(innerRing, centerpoint[0])
     innerCutout = solid.utils.down(jigThickness)(
         solid.linear_extrude(height=3 * jigThickness, convexity=10)(solid.polygon(innerRing)))
     registerRing = createOffsetPolygon(board, tolerance).exterior.coords
     if topSide:
-        print("top")
         registerRing = mirrorX(registerRing, centerpoint[0])
     registerCutout = solid.utils.up(jigThickness - pcbThickness)(
         solid.linear_extrude(height=jigThickness, convexity=10)(solid.polygon(registerRing)))
@@ -268,31 +265,34 @@ def renderScad(infile, outfile):
     outfile = os.path.abspath(outfile)
     subprocess.check_call(["openscad", "-o", outfile, infile])
 
+def shapelyToSHAPE_POLY_SET(polygon):
+    p = pcbnew.SHAPE_POLY_SET()
+    print(polygon.exterior)
+    p.AddOutline(linestringToKicad(polygon.exterior))
+    return p
+
+def cutoutComponents(board, components):
+    topCutout = extractComponentPolygons(components, "F.CrtYd")
+    for polygon in topCutout:
+        zone = pcbnew.PCB_SHAPE()
+        zone.SetShape(STROKE_T.S_POLYGON)
+        zone.SetPolyShape(shapelyToSHAPE_POLY_SET(polygon))
+        zone.SetLayer(Layer.F_Paste)
+        board.Add(zone)
+    bottomCutout = extractComponentPolygons(components, "B.CrtYd")
+    for polygon in bottomCutout:
+        zone = pcbnew.PCB_SHAPE()
+        zone.SetShape(STROKE_T.S_POLYGON)
+        zone.SetPolyShape(shapelyToSHAPE_POLY_SET(polygon))
+        zone.SetLayer(Layer.B_Paste)
+        board.Add(zone)
+
 
 from pathlib import Path
 import os
 
-@click.command()
-@click.argument("inputBoard", type=click.Path(dir_okay=False))
-@click.argument("outputDir", type=click.Path(dir_okay=True))
-@click.option("--jigsize", type=(int, int), default=(100, 100),
-    help="Jig frame size in mm: <width> <height>")
-@click.option("--jigthickness", type=float, default=3,
-    help="Jig thickness in mm")
-@click.option("--pcbthickness", type=float, default=1.6,
-    help="PCB thickness in mm")
-@click.option("--registerborder", type=(float, float), default=(3, 1),
-    help="Register borders in mm: <outer> <inner>")
-@click.option("--tolerance", type=float, default=0.05,
-    help="Enlarges the register by the tolerance value")
-@click.option("--ignore", type=str, default="",
-    help="Comma separated list of components references to exclude from the stencil")
 def create(inputboard, outputdir, jigsize, jigthickness, pcbthickness,
-           registerborder, tolerance, ignore):
-    """
-    Create stencil and register elements for manual paste dispensing jig.
-    See more details at: https://github.com/yaqwsx/KiKit/blob/master/doc/stencil.md
-    """
+           registerborder, tolerance, ignore, cutout):
     board = pcbnew.LoadBoard(inputboard)
     refs = parseReferences(ignore)
     removeComponents(board, refs)
@@ -301,6 +301,7 @@ def create(inputboard, outputdir, jigsize, jigthickness, pcbthickness,
 
     jigsize = (fromMm(jigsize[0]), fromMm(jigsize[1]))
     addJigFrame(board, jigsize)
+    cutoutComponents(board, getComponents(board, parseReferences(cutout)))
 
     stencilFile = os.path.join(outputdir, "stencil.kicad_pcb")
     board.Save(stencilFile)
@@ -343,7 +344,32 @@ def printedStencilSubstrate(outlineDxf, thickness, frameHeight, frameWidth, fram
             boardOffset(solid.import_dxf(outlineDxf))))
     return body - board
 
-def printedStencil(outlineDxf, holesDxf, thickness, frameHeight, frameWidth,
+def getComponents(board, references):
+    """
+    Return a list of components based on designator
+    """
+    return [f for f in board.GetFootprints() if f.GetReference() in references]
+
+def collectFootprintEdges(footprint, layerName):
+    """
+    Return all edges on given layer in given footprint
+    """
+    return [e for e in footprint.GraphicalItems() if e.GetLayerName() == layerName]
+
+def extractComponentPolygons(footprints, srcLayer):
+    """
+    Return a list of shapely polygons with holes for already placed components.
+    The source layer defines the geometry on which the cutout is computed.
+    Usually it a font or back courtyard
+    """
+    polygons = []
+    for f in footprints:
+       edges = collectFootprintEdges(f, srcLayer)
+       for ring in extractRings(edges):
+           polygons.append(toShapely(ring, edges))
+    return polygons
+
+def printedStencil(outlineDxf, holesDxf, extraHoles, thickness, frameHeight, frameWidth,
                    frameClearance, enlargeHoles, front):
     zScale = -1 if front else 1
     xRotate = 180 if front else 0
@@ -352,30 +378,21 @@ def printedStencil(outlineDxf, holesDxf, thickness, frameHeight, frameWidth,
     holesOffset = solid.utils.up(0) if enlargeHoles == 0 else solid.offset(delta=enlargeHoles)
     holes = solid.linear_extrude(height=4*thickness, center=True)(
         holesOffset(solid.import_dxf(holesDxf)))
-    return solid.rotate(a=xRotate, v=[1, 0, 0])(substrate - holes)
+    substrate -= holes
+    for h in extraHoles:
+        substrate -= solid.scale([toMm(1), -toMm(1), 1])(
+                solid.linear_extrude(height=4*thickness, center=True)(
+                    solid.polygon(h.exterior.coords)))
+    return solid.rotate(a=xRotate, v=[1, 0, 0])(substrate)
 
-@click.command()
-@click.argument("inputBoard", type=click.Path(dir_okay=False))
-@click.argument("outputDir", type=click.Path(dir_okay=True))
-@click.option("--pcbthickness", type=float, default=1.6,
-    help="PCB thickness in mm")
-@click.option("--thickness", type=float, default=0.15,
-    help="Stencil thickness in mm. Defines amount of paste dispensed")
-@click.option("--framewidth", type=float, default=1,
-    help="Register frame width")
-@click.option("--ignore", type=str, default="",
-    help="Comma separated list of components references to exclude from the stencil")
-@click.option("--frameclearance", type=float, default=0,
-    help="Clearance for the stencil register in milimeters")
-@click.option("--enlargeholes", type=float, default=0,
-    help="Enlarge pad holes by x mm")
 def createPrinted(inputboard, outputdir, pcbthickness, thickness, framewidth,
-                  ignore, frameclearance, enlargeholes):
+                  ignore, cutout, frameclearance, enlargeholes):
     """
     Create a 3D printed self-registering stencil.
     """
     board = pcbnew.LoadBoard(inputboard)
     refs = parseReferences(ignore)
+    cutoutComponents = getComponents(board, parseReferences(cutout))
     removeComponents(board, refs)
     Path(outputdir).mkdir(parents=True, exist_ok=True)
 
@@ -384,10 +401,12 @@ def createPrinted(inputboard, outputdir, pcbthickness, thickness, framewidth,
     # versions.
     height = min(pcbthickness, max(0.5, pcbthickness - 0.3))
     bottomPaste, topPaste, outline = pasteDxfExport(board, outputdir)
-    topStencil = printedStencil(outline, topPaste, thickness, height,
+    topCutout = extractComponentPolygons(cutoutComponents, "F.CrtYd")
+    bottomCutout = extractComponentPolygons(cutoutComponents, "B.CrtYd")
+    topStencil = printedStencil(outline, topPaste, topCutout, thickness, height,
         framewidth, frameclearance, enlargeholes, True)
-    bottomStencil = printedStencil(outline, bottomPaste, thickness, height,
-        framewidth, frameclearance, enlargeholes, False)
+    bottomStencil = printedStencil(outline, bottomPaste, bottomCutout, thickness,
+        height, framewidth, frameclearance, enlargeholes, False)
 
     bottomStencilFile = os.path.join(outputdir, "bottomStencil.scad")
     solid.scad_render_to_file(bottomStencil, bottomStencilFile,
@@ -400,11 +419,4 @@ def createPrinted(inputboard, outputdir, pcbthickness, thickness, framewidth,
     renderScad(topStencilFile, os.path.join(outputdir, "topStencil.stl"))
 
 
-@click.group()
-def stencil():
-    """
-    Create solder paste stencils
-    """
-    pass
-stencil.add_command(create)
-stencil.add_command(createPrinted)
+
